import unittest

import hcl2

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.aws.ALBListenerHTTPS import check


class TestALBListenerHTTPS(unittest.TestCase):

    def test_success_redirect(self):
        resource_conf = {'load_balancer_arn': ['${aws_lb.front_end.arn}'], 'port': ['80'], 'protocol': ['HTTP'],
                         'default_action': [{'type': ['redirect'], 'redirect': [
                             {'port': ['443'], 'protocol': ['HTTPS'], 'status_code': ['HTTP_301']}]}]}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_1(self):
        resource_conf = {'load_balancer_arn': ['${aws_lb.front_end.arn}'], 'port': ['443'], 'protocol': ['HTTPS']}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_2(self):
        resource_conf = {'load_balancer_arn': ['${aws_alb.front_end.arn}'], 'port': ['443'], 'protocol': ['HTTPS']}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_nlb_tcp_success(self):
        resource_conf = {'load_balancer_arn': ['${aws_lb.front_end.arn}'], 'port': ['22'], 'protocol': ['TCP']}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_nlb_udp_success(self):
        resource_conf = {'load_balancer_arn': ['${aws_lb.front_end.arn}'], 'port': ['53'], 'protocol': ['UDP']}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_nlb_tcp_udp_success(self):
        resource_conf = {'load_balancer_arn': ['${aws_lb.front_end.arn}'], 'port': ['53'], 'protocol': ['TCP_UDP']}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failure_1(self):
        resource_conf = {'load_balancer_arn': ['${aws_lb.front_end.arn}'], 'port': ['80'], 'protocol': ['HTTP']}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_2(self):
        resource_conf = {'load_balancer_arn': ['${aws_alb.front_end.arn}'], 'port': ['80'], 'protocol': ['HTTP']}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success_no_protocol(self):
        hcl_res = hcl2.loads("""
resource "aws_lb_listener" "http_redirector" {
  load_balancer_arn = aws_lb.redirector.arn
  port              = "80"
  protocol          = "HTTP"
  default_action {
    type = "redirect"
    redirect {
      host        = "example.com"
      status_code = "HTTP_302"
    }
  }
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_lb_listener']['http_redirector']
        result = check.scan_resource_conf(resource_conf)
        self.assertEqual(CheckResult.UNKNOWN, result)

    def test_unknown_not_rendered(self):
        hcl_res = hcl2.loads("""
resource "aws_lb_listener" "http_redirector" {
  load_balancer_arn = aws_lb.redirector.arn
  port              = "80"
  protocol          = var.lb_protocol
  default_action {
    type = "redirect"
    redirect {
      host        = "example.com"
      status_code = "HTTP_302"
    }
  }
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_lb_listener']['http_redirector']
        result = check.scan_resource_conf(resource_conf)
        self.assertEqual(CheckResult.UNKNOWN, result)


if __name__ == '__main__':
    unittest.main()



import json
import os
import unittest
from unittest import mock
from checkov.common.logger_streams import LoggerStreams
from checkov.logging_init import log_stream, erase_log_stream
from checkov.main import Checkov
from checkov.common.util.env_vars_config import env_vars_config

current_dir = os.path.dirname(os.path.realpath(__file__))


class TestCheckovConfig(unittest.TestCase):
    def test_terragoat_report(self):
        # Report to be generated using following command:
        # checkov -d path/to/terragoat --config-file \
        # path/to/checkov/integration_tests/example_config_files/config.yaml \
        # > path/to/checkov/checkov_config_report_terragoat.json
        report_path = os.path.join(os.path.dirname(current_dir), "checkov_config_report_terragoat.json")
        with open(report_path) as json_file:
            data = json.load(json_file)
            self.assertEqual(
                data["summary"]["parsing_errors"],
                0,
                f"expecting 0 parsing errors but got: {data['results']['parsing_errors']}",
            )
            self.assertGreater(
                data["summary"]["failed"], 1, f"expecting more than 1 failed checks, got: {data['summary']['failed']}"
            )
            self.assertEqual(
                data["check_type"], "terraform", f"expecting 'terraform' but got: {data['check_type']}"
            )
            self.assertIsNotNone(
                data["results"]["failed_checks"][0]["guideline"], "expecting a guideline for checks."
            )

    def setUp(self):
        erase_log_stream()  # Clear any existing logs before each test
        self.logger_streams = LoggerStreams()
        self.stream_name = "test_stream"
        self.logger_streams.add_stream(self.stream_name, log_stream)

    def tearDown(self):
        erase_log_stream()  # Ensure logs are cleared after each test

    def get_logged_messages(self):
        return self.logger_streams.get_streams().get(self.stream_name).getvalue()

    def test_missing_config_file(self):
        """Test when the provided config-file does not exist."""
        env_vars_config.ENABLE_CONFIG_FILE_VALIDATION = True
        config_path = os.path.join('path', 'to', 'missing', 'config.yaml')
        argv = ["--config-file", config_path]

        with mock.patch("pathlib.Path.is_file", return_value=False):
            checkov_instance = Checkov(argv=argv)
            checkov_instance.parse_config()

        logged_messages = self.get_logged_messages()
        expected_message = f"The config file at '{config_path}' does not exist. Running without a config file."
        self.assertIn(expected_message, logged_messages)

    def test_no_config_file_argument(self):
        """Test when no --config-file argument is provided."""
        env_vars_config.ENABLE_CONFIG_FILE_VALIDATION = True
        argv = []

        checkov_instance = Checkov(argv=argv)
        checkov_instance.parse_config()

        logged_messages = self.get_logged_messages()
        self.assertNotIn("does not exist", logged_messages)


if __name__ == "__main__":
    unittest.main()



import argparse

import pytest

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.runners.runner_registry import RunnerRegistry


@pytest.mark.parametrize(
    "soft_fail,soft_fail_checks,hard_fail_checks,expected_exit_code",
    [
        (True, [], ['Valid'], 0),  # Soft fail overrides any check-specific definition
        (True, ['Invalid'], ['Valid'], 0),  # Soft fail overrides any check-specific definition
        (False, [], ['Valid'], 1),  # Hard fail on valid
        (False, ['Valid'], ['Valid'], 1),  # Hard fail check wins same soft fail check
        (False, [], ['Invalid'], 1),  # Hard fail on invalid
        (False, [], ['Unknown'], 1),  # Hard fail on unknown
        (False, ['Invalid', 'Valid', 'Unknown'], [], 0),  # Implicit soft fail on all statuses
        (True, [], ['Invalid', 'Valid', 'Unknown'], 0),  # Soft fail wins Implicit hard fail on all statuses
        (False, [], ['Invalid', 'Valid', 'Unknown'], 1),  # Implicit hard fail on all statuses
        (False, [], [], 1),  # default
        (True, [], [], 0)  # soft fail
    ],
)
def test_secrets_get_exit_code(secrets_report, soft_fail, soft_fail_checks, hard_fail_checks, expected_exit_code) -> None:
    exit_code_thresholds = {'soft_fail': soft_fail, 'soft_fail_checks': soft_fail_checks,
                            'soft_fail_threshold': None, 'hard_fail_checks': hard_fail_checks,
                            'hard_fail_threshold': None}

    assert secrets_report.get_exit_code(exit_code_thresholds) == expected_exit_code


@pytest.mark.parametrize(
    "soft_fail,soft_fail_on,hard_fail_on,expected_soft_fail_checks,expected_hard_fail_checks",
    [
        (False, None, None, [], []),  # default
        (False, 'invalid', None, ['Invalid'], []),  # assigning correct casing
        (False, 'invalid,valid,unknown', 'invalid,valid,unknown', ['Invalid', 'Valid', 'Unknown'], ['Invalid', 'Valid', 'Unknown'])  # assigning correct casing
    ]
)
def test_secrets_get_fail_threshold(soft_fail, soft_fail_on, hard_fail_on,
                                    expected_soft_fail_checks, expected_hard_fail_checks) -> None:
    config = argparse.Namespace(
        soft_fail=soft_fail,
        soft_fail_on=soft_fail_on,
        hard_fail_on=hard_fail_on,
        use_enforcement_rules=False
    )

    expected = {
        'soft_fail': soft_fail,
        'soft_fail_checks': expected_soft_fail_checks,
        'soft_fail_threshold': None,
        'hard_fail_checks': expected_hard_fail_checks,
        'hard_fail_threshold': None
    }

    assert RunnerRegistry.get_fail_thresholds(config, report_type=CheckType.SECRETS) == expected



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.DataLakeStoreEncryption import check
from checkov.common.models.enums import CheckResult


class TestDataLakeStoreEncryption(unittest.TestCase):

    def test_failure_explicit(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_data_lake_store" "example" {
              name                = "consumptiondatalake"
              location            = azurerm_resource_group.example.location
              resource_group_name = azurerm_resource_group.example.name
              
              encryption_state = "Disabled"
              }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_data_lake_store']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_data_lake_store" "example" {
              name                = "consumptiondatalake"
              location            = azurerm_resource_group.example.location
              resource_group_name = azurerm_resource_group.example.name
              }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_data_lake_store']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_explicit(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_data_lake_store" "example" {
              name                = "consumptiondatalake"
              location            = azurerm_resource_group.example.location
              resource_group_name = azurerm_resource_group.example.name
              
              encryption_state = "Enabled"
              }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_data_lake_store']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from __future__ import annotations
import pytest

from checkov.common.images.image_referencer import Image


@pytest.fixture
def file_path() -> str:
    return ".circleci/config.yml"


@pytest.fixture
def definition(file_path) -> dict:
    return {
            file_path: {
              "orbs": {
                "new-orb": "whatever/orbname@goodorb",
                "some-orb": "orbs/orbname@dev:blah",
                "__startline__": 3,
                "__endline__": 6
              },
              "executors": {
                "image-executor": {
                  "docker": [
                    {
                      "image": "mongo:2.6.8",
                      "__startline__": 9,
                      "__endline__": 11
                    }
                  ],
                  "__startline__": 8,
                  "__endline__": 11
                },
                "__startline__": 7,
                "__endline__": 11
              },
              "jobs": {
                "test-docker-hash-img": {
                  "docker": [
                    {
                      "image": "redis@sha256:54057dd7e125ca41afe526a877e8bd35ec2cdd33b9217e022ed37bdcf7d09673",
                      "__startline__": 15,
                      "__endline__": 16
                    },
                    {
                      "auth": {
                        "password": "$DOCKERHUB_PASSWORD",
                        "username": "mydockerhub-user",
                        "__startline__": 18,
                        "__endline__": 20
                      },
                      "__startline__": 17,
                      "__endline__": 20
                    }
                  ],
                  "__startline__": 13,
                  "__endline__": 20
                },
                "test-docker-latest-img": {
                  "docker": [
                    {
                      "image": "buildpack-deps:latest",
                      "__startline__": 23,
                      "__endline__": 24
                    },
                    {
                      "auth": {
                        "password": "$DOCKERHUB_PASSWORD",
                        "username": "mydockerhub-user",
                        "__startline__": 26,
                        "__endline__": 28
                      },
                      "command": [
                        "--smallfiles"
                      ],
                      "__startline__": 25,
                      "__endline__": 30
                    }
                  ],
                  "__startline__": 21,
                  "__endline__": 30
                },
                "test-docker-versioned-img": {
                  "docker": [
                    {
                      "image": "mongo:2.6.8",
                      "__startline__": 33,
                      "__endline__": 34
                    },
                    {
                      "auth": {
                        "password": "$DOCKERHUB_PASSWORD",
                        "username": "mydockerhub-user",
                        "__startline__": 36,
                        "__endline__": 38
                      },
                      "environment": {
                        "POSTGRES_USER": "user",
                        "__startline__": 39,
                        "__endline__": 40
                      },
                      "image": "postgres:14.2",
                      "__startline__": 35,
                      "__endline__": 41
                    }
                  ],
                  "__startline__": 31,
                  "__endline__": 41
                },
                "test-echo": {
                  "docker": [
                    {
                      "image": "cimg/python:latest",
                      "__startline__": 44,
                      "__endline__": 45
                    }
                  ],
                  "steps": [
                    "checkout",
                    {
                      "run": "echo \"this is an echo in a script.\"",
                      "__startline__": 48,
                      "__endline__": 49
                    }
                  ],
                  "__startline__": 42,
                  "__endline__": 49
                },
                "test-inject": {
                  "docker": [
                    {
                      "image": "cimg/python:latest",
                      "__startline__": 52,
                      "__endline__": 53
                    }
                  ],
                  "steps": [
                    "checkout",
                    {
                      "run": {
                        "command": "curl -sSJL https://www.mongodb.org/static/pgp/server-4.2.asc | sudo apt-key add -\necho ${CIRCLE_BRANCH}\n",
                        "name": "Multi-line run with injection via vars",
                        "__startline__": 56,
                        "__endline__": 60
                      },
                      "__startline__": 55,
                      "__endline__": 60
                    }
                  ],
                  "__startline__": 50,
                  "__endline__": 60
                },
                "test-inject2": {
                  "docker": [
                    {
                      "image": "cimg/python:latest",
                      "__startline__": 63,
                      "__endline__": 64
                    }
                  ],
                  "steps": [
                    "checkout",
                    {
                      "run": {
                        "command": "curl -sSJL https://www.mongodb.org/static/pgp/server-4.2.asc | sudo apt-key add -\necho $CIRCLE_BRANCH\n",
                        "name": "Multi-line run with injection via vars",
                        "__startline__": 68,
                        "__endline__": 72
                      },
                      "__startline__": 67,
                      "__endline__": 72
                    }
                  ],
                  "__startline__": 61,
                  "__endline__": 72
                },
                "test-curl-secret": {
                  "docker": [
                    {
                      "image": "cimg/python:latest",
                      "__startline__": 75,
                      "__endline__": 76
                    }
                  ],
                  "steps": [
                    "checkout",
                    {
                      "run": {
                        "command": "curl -x POST someurl $SECRET\n",
                        "name": "Multi-line export secret",
                        "__startline__": 79,
                        "__endline__": 82
                      },
                      "__startline__": 78,
                      "__endline__": 82
                    }
                  ],
                  "__startline__": 73,
                  "__endline__": 82
                },
                "test-inject-ci-vars": {
                  "docker": [
                    {
                      "image": "cimg/python:latest",
                      "__startline__": 85,
                      "__endline__": 86
                    }
                  ],
                  "steps": [
                    "checkout",
                    {
                      "run": {
                        "command": "echo ${CIRCLE_PR_REPONAME}\n",
                        "name": "Echo the PR Reponame",
                        "__startline__": 90,
                        "__endline__": 94
                      },
                      "__startline__": 89,
                      "__endline__": 94
                    }
                  ],
                  "__startline__": 83,
                  "__endline__": 94
                },
                "__startline__": 12,
                "__endline__": 94
              },
              "version": 2.1,
              "workflows": {
                "say-hello-workflow": {
                  "jobs": [
                    "test-docker-hash-img",
                    "test-docker-latest-img",
                    "test-docker-versioned-img",
                    "test-echo",
                    "test-inject",
                    "test-inject2",
                    "test-inject-ci-vars"
                  ],
                  "__startline__": 97,
                  "__endline__": 105
                },
                "__startline__": 96,
                "__endline__": 105
              },
              "__startline__": 2,
              "__endline__": 105
            }
          }


from __future__ import annotations

from pathlib import Path

import pytest
from pytest_mock import MockerFixture

from checkov.common.bridgecrew.bc_source import get_source_type
from checkov.common.bridgecrew.check_type import CheckType
from checkov.helm.runner import Runner
from checkov.runner_filter import RunnerFilter
from tests.common.image_referencer.test_utils import (
    mock_get_empty_license_statuses_async,
    mock_get_image_cached_result_async,
)
from tests.helm.utils import helm_exists

RESOURCES_PATH = Path(__file__).parent / "runner/resources"


@pytest.mark.skipif(not helm_exists(), reason="helm not installed")
def test_deployment_resources(mocker: MockerFixture):
    from checkov.common.bridgecrew.platform_integration import bc_integration

    # given
    file_name = "hello-world/templates/deployment.yaml"
    image_name = "nginx:1.16.0"
    code_lines = "20-42"
    test_folder = RESOURCES_PATH / "image_referencer"
    runner_filter = RunnerFilter(run_image_referencer=True)
    bc_integration.bc_source = get_source_type("disabled")

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_empty_license_statuses_async,
    )

    # when
    reports = Runner().run(root_folder=str(test_folder), runner_filter=runner_filter)

    # then
    assert len(reports) == 2

    helm_report = next(report for report in reports if report.check_type == CheckType.HELM)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(helm_report.resources) == 4
    assert len(helm_report.passed_checks) == 72
    assert len(helm_report.failed_checks) == 20
    assert len(helm_report.skipped_checks) == 0
    assert len(helm_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 1
    assert sca_image_report.resources == {
        f"{file_name} ({image_name} lines:{code_lines} (sha256:2460522297)).go",
    }
    assert len(sca_image_report.passed_checks) == 0
    assert len(sca_image_report.failed_checks) == 3
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0
    assert len(sca_image_report.image_cached_results) == 1

    assert sca_image_report.image_cached_results[0]["dockerImageName"] == image_name
    assert (
        sca_image_report.image_cached_results[0]["relatedResourceId"]
        == "/hello-world/templates/deployment.yaml:Pod.default.release-name-hello-world.app.kubernetes.io/name-hello-world.app.kubernetes.io/instance-release-name"
    )
    assert sca_image_report.image_cached_results[0]["packages"] == [
        {"type": "os", "name": "tzdata", "version": "2021a-1+deb11u5", "licenses": []}
    ]



import os
from pathlib import Path
from unittest import TestCase
from unittest import mock

from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.common.graph.graph_builder import EncryptionValues, EncryptionTypes
from checkov.common.graph.graph_builder.utils import calculate_hash
from checkov.common.graph.graph_builder.graph_components.attribute_names import CustomAttributes
from checkov.common.util.parser_utils import TERRAFORM_NESTED_MODULE_PATH_PREFIX, TERRAFORM_NESTED_MODULE_PATH_ENDING, \
    TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR
from checkov.terraform import TFModule
from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_builder.graph_components.blocks import TerraformBlock
from checkov.terraform.graph_builder.graph_components.generic_resource_encryption import ENCRYPTION_BY_RESOURCE_TYPE
from checkov.terraform.graph_builder.graph_to_tf_definitions import convert_graph_vertices_to_tf_definitions
from checkov.terraform.graph_builder.local_graph import TerraformLocalGraph
from checkov.terraform.graph_manager import TerraformGraphManager
from checkov.terraform.tf_parser import TFParser
from checkov.terraform.modules.module_utils import clean_parser_types, serialize_definitions

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


class TestLocalGraph(TestCase):
    def setUp(self) -> None:
        self.source = "TERRAFORM"

    def test_update_vertices_configs_attribute_like_resource_name(self):
        config = {"resource_type": {"resource_name": {"attribute1": 1, "attribute2": 2, "resource_name": ["caution!"]}}}
        attributes = {"attribute1": 1, "attribute2": 2, "resource_name": "ok"}
        local_graph = TerraformLocalGraph(None)
        vertex = TerraformBlock(name="resource_type.resource_name", config=config, path='', block_type=BlockType.RESOURCE, attributes=attributes)
        vertex.changed_attributes["resource_name"] = ""
        local_graph.vertices.append(vertex)
        local_graph.update_vertices_configs()
        expected_config = {"resource_type": {"resource_name": {"attribute1": 1, "attribute2": 2, "resource_name": ["ok"]}}}
        self.assertDictEqual(expected_config, vertex.config)

    def test_single_edge_with_same_label(self):
        resources_dir = os.path.realpath(
            os.path.join(TEST_DIRNAME, '../resources/k8_service'))

        graph_manager = TerraformGraphManager(NetworkxConnector())
        local_graph, _ = graph_manager.build_graph_from_source_directory(resources_dir,
                                                                          render_variables=True)
        edges_hash = []
        for e in local_graph.edges:
            edge_hash = calculate_hash({"origin": e.origin, "dest": e.dest, "label": e.label})
            if edge_hash in edges_hash:
                origin = local_graph.vertices[e.origin]
                dest = local_graph.vertices[e.dest]
                self.fail(f'edge {e} == [{origin} - {e.label} -> {dest}] appears more than once in the graph')
            else:
                edges_hash.append(edge_hash)

    @mock.patch.dict(os.environ, {"CHECKOV_NEW_TF_PARSER": "False"})
    def test_set_variables_values_from_modules(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME,
                                                      '../resources/variable_rendering/render_from_module_vpc'))
        hcl_config_parser = TFParser()
        module, _ = hcl_config_parser.parse_hcl_module(resources_dir, source=self.source)
        local_graph = TerraformLocalGraph(module)
        local_graph._create_vertices()

        variables_before_module_definitions = {
            "cidr": "0.0.0.0/0",
            "private_subnets": [],
            "public_subnets": [],
            "enable_nat_gateway": False,
            "single_nat_gateway": False,
            "enable_dns_hostnames": False,
            "public_subnet_tags": {},
            "private_subnet_tags": {},
        }

        for var_name, var_value in variables_before_module_definitions.items():
            vertex_index = local_graph.vertices_block_name_map[BlockType.VARIABLE].get(var_name)[0]
            vertex = local_graph.vertices[vertex_index]
            default_val = vertex.attributes['default']
            if type(default_val) == list:
                self.assertEqual(var_value, default_val[0])
            else:
                self.assertEqual(var_value, default_val)

        local_graph.build_graph(resources_dir)

        expected_variables_after = {
            "cidr": "172.16.0.0/16",
            "private_subnets": ["172.16.1.0/24", "172.16.2.0/24", "172.16.3.0/24"],
            "public_subnets": ["172.16.4.0/24", "172.16.5.0/24", "172.16.6.0/24"],
            "enable_nat_gateway": True,
            "single_nat_gateway": True,
            "enable_dns_hostnames": True,
            "public_subnet_tags": {"kubernetes.io/cluster/${local.cluster_name}": "shared",
                                    "kubernetes.io/role/elb": "1"},
            "private_subnet_tags": {"kubernetes.io/cluster/${local.cluster_name}": "shared",
                                    "kubernetes.io/role/internal-elb": "1"}
        }

        for var_name, var_value in expected_variables_after.items():
            vertex_index = local_graph.vertices_block_name_map[BlockType.VARIABLE].get(var_name)[0]
            vertex = local_graph.vertices[vertex_index]
            default_val = vertex.attributes['default']
            if type(default_val) == list:
                self.assertEqual(var_value, default_val[0])
            else:
                self.assertEqual(var_value, default_val)

    def test_definition_creation_by_dirs(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME,
                                                      '../resources/variable_rendering/render_local_from_variable'))
        hcl_config_parser = TFParser()
        tf_definitions = hcl_config_parser.parse_directory(directory=resources_dir)
        tf_definitions = clean_parser_types(tf_definitions)
        tf_definitions = serialize_definitions(tf_definitions)

        dirs_to_definitions = hcl_config_parser.create_definition_by_dirs(tf_definitions)
        assert len(dirs_to_definitions) == 1
        single_dir_element = list(dirs_to_definitions.values())[0]
        assert list(single_dir_element[0].values()) == [{'locals': [{'__end_line__': 3, '__start_line__': 1, 'bucket_name': ['${var.var_bucket_name}']}]}]
        assert list(single_dir_element[1].values()) == [{'variable': [{'var_bucket_name': {'__end_line__': 3, '__start_line__': 1, 'default': ['test_bucket_name']}}]}]

    def test_definition_creation_by_dirs_multi_nodule(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME,
                                                      '../resources/modules/linked_modules'))
        hcl_config_parser = TFParser()
        tf_definitions = hcl_config_parser.parse_directory(directory=resources_dir)
        tf_definitions = clean_parser_types(tf_definitions)
        tf_definitions = serialize_definitions(tf_definitions)

        dirs_to_definitions = hcl_config_parser.create_definition_by_dirs(tf_definitions)
        assert len(dirs_to_definitions) == 2
        lambda_element = list(dirs_to_definitions.values())[0]
        s3_bucket_element = list(dirs_to_definitions.values())[1]
        assert len(lambda_element) + len(s3_bucket_element) == len(tf_definitions)
        modules = hcl_config_parser.parse_multi_graph_hcl_module(resources_dir, source=self.source)
        assert len(modules) == 2
        assert 'lambda' in modules[0][0].source_dir
        assert 's3-bucket' in modules[1][0].source_dir


    def test_compare_multi_graph_defs(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME,
                                                      '../resources/variable_rendering/render_module_postgresql'))
        hcl_config_parser = TFParser()
        module, defs = hcl_config_parser.parse_hcl_module(resources_dir, source=self.source)
        modules = hcl_config_parser.parse_multi_graph_hcl_module(resources_dir, source=self.source)
        for idx, module_to_definitions in enumerate(modules):
            assert module_to_definitions[0] == module
            for att, content in defs.items():
                found = False
                for content_dict in module_to_definitions[1]:
                    for key, value in content_dict.items():
                        if value == content:
                            found = True
                            break
                    if found:
                        break
                assert found

    def test_set_variables_values_from_modules_with_new_tf_parser(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME,
                                                      '../resources/variable_rendering/render_from_module_vpc'))
        hcl_config_parser = TFParser()
        module, _ = hcl_config_parser.parse_hcl_module(resources_dir, source=self.source)
        local_graph = TerraformLocalGraph(module)
        local_graph._create_vertices()

        variables_before_module_definitions = {
            "cidr": "0.0.0.0/0",
            "private_subnets": [],
            "public_subnets": [],
            "enable_nat_gateway": False,
            "single_nat_gateway": False,
            "enable_dns_hostnames": False,
            "public_subnet_tags": {},
            "private_subnet_tags": {},
        }

        for var_name, var_value in variables_before_module_definitions.items():
            vertex_index = local_graph.vertices_block_name_map[BlockType.VARIABLE].get(var_name)[0]
            vertex = local_graph.vertices[vertex_index]
            default_val = vertex.attributes['default']
            if type(default_val) == list:
                self.assertEqual(var_value, default_val[0])
            else:
                self.assertEqual(var_value, default_val)

        local_graph.build_graph(resources_dir)

        expected_variables_after = {
            "cidr": "172.16.0.0/16",
            "private_subnets": ["172.16.1.0/24", "172.16.2.0/24", "172.16.3.0/24"],
            "public_subnets": ["172.16.4.0/24", "172.16.5.0/24", "172.16.6.0/24"],
            "enable_nat_gateway": True,
            "single_nat_gateway": True,
            "enable_dns_hostnames": True,
            "public_subnet_tags": {"kubernetes.io/cluster/${local.cluster_name}": "shared",
                                    "kubernetes.io/role/elb": "1"},
            "private_subnet_tags": {"kubernetes.io/cluster/${local.cluster_name}": "shared",
                                    "kubernetes.io/role/internal-elb": "1"}
        }

        for var_name, var_value in expected_variables_after.items():
            vertex_index = local_graph.vertices_block_name_map[BlockType.VARIABLE].get(var_name)[0]
            vertex = local_graph.vertices[vertex_index]
            default_val = vertex.attributes['default']
            if type(default_val) == list:
                self.assertEqual(var_value, default_val[0])
            else:
                self.assertEqual(var_value, default_val)

    def test_encryption_aws(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/encryption'))
        hcl_config_parser = TFParser()
        module, _ = hcl_config_parser.parse_hcl_module(resources_dir, self.source)
        local_graph = TerraformLocalGraph(module)
        local_graph._create_vertices()
        local_graph.calculate_encryption_attribute(ENCRYPTION_BY_RESOURCE_TYPE)
        all_attributes = [vertex.get_attribute_dict() for vertex in local_graph.vertices]
        for attribute_dict in all_attributes:
            [resource_type, resource_name] = attribute_dict[CustomAttributes.ID].split(".")
            if resource_type in ENCRYPTION_BY_RESOURCE_TYPE:
                is_encrypted = attribute_dict[CustomAttributes.ENCRYPTION]
                details = attribute_dict[CustomAttributes.ENCRYPTION_DETAILS]
                self.assertEqual(is_encrypted, EncryptionValues.ENCRYPTED.value if resource_name.startswith("encrypted")
                                 else EncryptionValues.UNENCRYPTED.value, f'failed for "{resource_type}.{resource_name}"')
                if is_encrypted == EncryptionValues.ENCRYPTED.value:
                    if 'kms_key_id' in attribute_dict or 'kms_master_key_id' in attribute_dict:
                        self.assertEqual(details, EncryptionTypes.KMS_VALUE.value, f'Bad encryption details for "{resource_type}.{resource_name}"')
                    else:
                        self.assertIn(details, [EncryptionTypes.AES256.value, EncryptionTypes.KMS_VALUE.value, EncryptionTypes.NODE_TO_NODE.value, EncryptionTypes.DEFAULT_KMS.value], f'Bad encryption details for "{resource_type}.{resource_name}"')
                else:
                    self.assertEqual(details, "")
            else:
                self.assertIsNone(attribute_dict.get(CustomAttributes.ENCRYPTION))
                self.assertIsNone(attribute_dict.get(CustomAttributes.ENCRYPTION_DETAILS))

    def test_vertices_from_local_graph(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME,
                                                      '../resources/variable_rendering/render_from_module_vpc'))
        hcl_config_parser = TFParser()
        module, _ = hcl_config_parser.parse_hcl_module(resources_dir, self.source)
        local_graph = TerraformLocalGraph(module)
        local_graph._create_vertices()
        tf_definitions, breadcrumbs = convert_graph_vertices_to_tf_definitions(local_graph.vertices, resources_dir)
        self.assertIsNotNone(tf_definitions)
        self.assertIsNotNone(breadcrumbs)

    def test_blocks_from_local_graph_module(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/modules/stacks'))
        hcl_config_parser = TFParser()
        module, _ = hcl_config_parser.parse_hcl_module(resources_dir, self.source)
        self.assertEqual(len(list(filter(lambda block: block.block_type == BlockType.RESOURCE and block.name == 'aws_s3_bucket.inner_s3', module.blocks))), 3)
        self.assertEqual(len(list(filter(lambda block: block.block_type == BlockType.MODULE and block.name == 'inner_module_call', module.blocks))), 3)
        self.assertEqual(len(list(filter(lambda block: block.block_type == BlockType.MODULE and block.name == 's3', module.blocks))), 3)
        self.assertEqual(len(list(filter(lambda block: block.block_type == BlockType.MODULE and block.name == 'sub-module', module.blocks))), 1)

    def test_vertices_from_local_graph_module(self):
        parent_dir = Path(TEST_DIRNAME).parent
        resources_dir = str(parent_dir / "resources/modules/stacks")
        hcl_config_parser = TFParser()
        module, _ = hcl_config_parser.parse_hcl_module(resources_dir, self.source)
        local_graph = TerraformLocalGraph(module)
        local_graph.build_graph(render_variables=True)

        self.assertEqual(12, len(local_graph.edges))

        # check vertex breadcrumbs
        bucket_vertex_1 = next(
            vertex
            for vertex in local_graph.vertices
            if vertex.name == "aws_s3_bucket.inner_s3" and vertex.source_module == {6}
        )
        bucket_vertex_2 = next(
            vertex
            for vertex in local_graph.vertices
            if vertex.name == "aws_s3_bucket.inner_s3" and vertex.source_module == {7}
        )
        bucket_vertex_3 = next(
            vertex
            for vertex in local_graph.vertices
            if vertex.name == "aws_s3_bucket.inner_s3" and vertex.source_module == {8}
        )

        expected_vertex_1_breadcrumbs = {
                "versioning.enabled": [
                    {
                        "type": "module",
                        "name": "inner_module_call",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/main.tf"),
                        "module_connection": False,
                    },
                    {
                        "type": "variable",
                        "name": "versioning",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/inner/variables.tf"),
                        "module_connection": False,
                    },
                ],
                "source_module_": [
                    {
                        'type': 'module',
                        'name': 'sub-module',
                        'path': str(parent_dir / 'resources/modules/stacks/prod/main.tf'),
                        'idx': 12,
                        'source_module_object': None
                    }
                    ,
                    {
                        'type': 'module',
                        'name': 's3',
                        'path': str(parent_dir / 'resources/modules/stacks/prod/sub-prod/main.tf'),
                        'idx': 13,
                        'source_module_object': TFModule(path=str(parent_dir / 'resources/modules/stacks/prod/main.tf'),
                                                         name='sub-module', foreach_idx=None, nested_tf_module=None)
                    }
                    ,
                    {
                        'type': 'module',
                        'name': 'inner_module_call',
                        'path': str(parent_dir / 'resources/modules/s3_inner_modules/main.tf'),
                        'idx': 6,
                        'source_module_object': TFModule(path=str(parent_dir / 'resources/modules/stacks/prod/sub-prod/main.tf'),
                                                         name='s3', foreach_idx=None,
                                                         nested_tf_module=TFModule(path=str(parent_dir / 'resources/modules/stacks/prod/main.tf'),
                                                                                   name='sub-module', foreach_idx=None,
                                                                                   nested_tf_module=None)
                                                         )
                    }
                ],
            }

        self.assertDictEqual(expected_vertex_1_breadcrumbs, bucket_vertex_1.breadcrumbs)

        expected_vertex_2_breadcrumbs = {
                "versioning.enabled": [
                    {
                        "type": "module",
                        "name": "inner_module_call",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/main.tf"),
                        "module_connection": False,
                    },
                    {
                        "type": "variable",
                        "name": "versioning",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/inner/variables.tf"),
                        "module_connection": False,
                    },
                ],
                "source_module_": [
                    {
                        'type': 'module',
                        'name': 's3',
                        'path': str(parent_dir / 'resources/modules/stacks/stage/main.tf'),
                        'idx': 14,
                        'source_module_object': None
                    },
                    {
                        'type': 'module',
                        'name': 'inner_module_call',
                        'path': str(parent_dir / 'resources/modules/s3_inner_modules/main.tf'),
                        'idx': 7,
                        'source_module_object': TFModule(path=str(parent_dir / 'resources/modules/stacks/stage/main.tf'),
                                                         name='s3', foreach_idx=None, nested_tf_module=None)
                    }
                ],
            }
        self.assertDictEqual(expected_vertex_2_breadcrumbs, bucket_vertex_2.breadcrumbs)

        expected_vertex_3_breadcrumbs = {
                "versioning.enabled": [
                    {
                        "type": "module",
                        "name": "inner_module_call",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/main.tf"),
                        "module_connection": False,
                    },
                    {
                        "type": "variable",
                        "name": "versioning",
                        "path": str(parent_dir / "resources/modules/s3_inner_modules/inner/variables.tf"),
                        "module_connection": False,
                    },
                ],
                "source_module_": [
                    {
                        'type': 'module',
                        'name': 's3',
                        'path': str(parent_dir / 'resources/modules/stacks/test/main.tf'),
                        'idx': 15,
                        'source_module_object': None
                    },
                    {
                        'type': 'module',
                        'name': 'inner_module_call',
                        'path': str(parent_dir / 'resources/modules/s3_inner_modules/main.tf'),
                        'idx': 8,
                        'source_module_object': TFModule(path=str(parent_dir / 'resources/modules/stacks/test/main.tf'),
                                                         name='s3', foreach_idx=None, nested_tf_module=None)
                    }
                ],
        }
        self.assertDictEqual(expected_vertex_3_breadcrumbs, bucket_vertex_3.breadcrumbs)

    def test_variables_same_name_different_modules(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/modules/same_var_names'))
        hcl_config_parser = TFParser()
        module, _ = hcl_config_parser.parse_hcl_module(resources_dir, self.source)
        local_graph = TerraformLocalGraph(module)
        local_graph.build_graph(render_variables=True)
        print(local_graph.edges)
        self.assertEqual(12, len(local_graph.edges))
        self.assertEqual(13, len(local_graph.vertices))

        module_variable_edges = [
            e for e in local_graph.edges
            if local_graph.vertices[e.dest].block_type == "module" and local_graph.vertices[e.dest].path.endswith(
                'same_var_names/module2/main.tf')
        ]

        # Check they point to 2 different modules
        self.assertEqual(2, len(module_variable_edges))
        self.assertNotEqual(local_graph.vertices[module_variable_edges[0].origin],
                            local_graph.vertices[module_variable_edges[1].origin])


        module_variable_edges = [
            e for e in local_graph.edges
            if local_graph.vertices[e.dest].block_type == "module" and local_graph.vertices[e.dest].path.endswith('same_var_names/module1/main.tf')
        ]

        # Check they point to 2 different modules
        self.assertEqual(2, len(module_variable_edges))
        self.assertNotEqual(local_graph.vertices[module_variable_edges[0].origin], local_graph.vertices[module_variable_edges[1].origin])
